#nullable disable
using Newtonsoft.Json;

namespace FitConnect.Models.v1.Api;

public class Problems {
    // @formatter:off
    public const string TitleEventLogInconsistent = "Inkonsistentes Event-Log";
    public const string DetailEventLogInconsistent = "Das Event-Log ist inkonsistent.";
    public const string DetailEventLogNotExactlyOneSubmit = "Das Event-Log ist inkonsistent, da es nicht genau ein Event 'submit-submission' enthält.";

    public const string TitleAuthenticationTagMissing = "Fehlende Authentication-Tags";
    public const string TitleAuthenticationTagInvalid = "Authentication-Tag ungültig";
    public const string DetailAuthenticationTagMissing = "Das Event 'submit-submission' enthält keine Authentication-Tags.";
    public const string DetailAuthenticationTagDataInvalid = "Das Authentication-Tag des Fachdatensatzes ist ungültig.";
    public const string DetailAuthenticationMetadataInvalid = "Das Authentication-Tag des Metadatensatzes ist ungültig.";
    public const string DetailAuthenticationAttachmentInvalid = "Das Authentication-Tag der Anlage {attachmentId} ist ungültig.";
    
    public const string TitleEncryptionIssue = "Entschlüsselungs-Fehler";
    public const string DetailEncryptionIssueMetadata = "Die Entschlüsselung des Metadatensatzes ist fehlgeschlagen.";
    public const string DetailEncryptionIssueData = "Der Fachdatensatz konnte nicht entschlüsselt werden.";
   public const string DetailEncryptionIssueAttachment = "Die Anlage {0} konnte nicht entschlüsselt werden.";
    
    public const string TitleMissingData = "Fachdatensatz fehlt";
    public const string DetailMissingData = "Der Fachdatensatz fehlt.";

    public const string TitleHashMismatch = "Prüfsumme stimmt nicht";
    public const string DetailDataHashMismatch = "Die Prüfsumme des Fachdatensatzes stimmt nicht.";

    public const string TitleMissingSchema = "Schema-Referenz fehlt";
    public const string DetailMissingSchema = "Die Schema-Referenz fehlt im Metadatensatz.";

    public const string TitleSchemaViolationData = "Schema-Fehler";
    public const string DetailsSchemaViolationData = "Der Fachdatensatz ist nicht schema-valide.";

    public const string TitleUnsupportedService = "Verwaltungsleistung nicht unterstützt";
    public const string DetailUnsupportedService = "Die angegebene Verwaltungsleistung wird nicht unterstützt.";
    
    public const string TitleSyntaxViolation = "Syntax-Fehler";
    public const string DetailSyntaxViolationMetadata = "Der Metadatensatz ist kein valides JSON.";
    public const string DetailSyntaxViolationDataJson = "Der Fachdatensatz ist kein valides JSON.";
    public const string DetailSyntaxViolationDataXml = "Der Fachdatensatz ist kein valides XML.";
    
    public const string TitleServiceMismatch = "Verwaltungsleistung stimmt nicht überein";
    public const string DetailServiceMismatch = "Die Verwaltungsleistung in Submission und Metadatensatz stimmen nicht überein.";
    
    
    public const string TitleAttachmentMissing = "Fehlerhafte Anlagen-Liste";
    public const string DetailAttachmentMissing = "Die Anlage {0} konnte nicht geladen werden.";
    
    public const string TitleAttachmentsMismatch = "Fehlerhafte Anlagen-Liste";
    public const string DetailAttachmentsMismatch = "Die Liste der Anlagen in Submission und Event-Log stimmt nicht überein.";
    
    public const string TitleUnsupportedSchema = "Metadatenschema nicht unterstützt";
    public const string DetailUnsupportedSchema = "Die angegebene Metadatenschema-URI ('$schema') ist keines der unterstützten Metadatenschemas.";

    public const string TitleUnsupportedDataSchema = "Fachdatenschema nicht unterstützt";
    public const string DetailUnsupportedDataSchema = "Das angegebene Fachdatenschema wird nicht unterstützt.";
    // @formatter:on

    public enum ProblemTypeEnum {
        AttachmentMismatch,
        MissingAttachments,
        EncryptionIssue,
        HashMismatch,
        IncorrectAuthenticationTag,
        InvalidEventLog,
        MissingAuthenticationTag,
        MissingData,
        MissingSchema,
        ServiceMismatch,
        SchemaViolation,
        SyntaxViolation,
        UnsupportedDataSchema,
        UnsupportedMetaSchema,
        UnsupportedService
    }

    public enum ProblemInstanceEnum {
        Submission,
        Metadata,
        Data
    }

    internal Problems() {
    }

    public Problems(ProblemTypeEnum problemType, string detail) {
        type = "https://schema.fitko.de/fit-connect/events/problems/";
        switch (problemType) {
            case ProblemTypeEnum.MissingAttachments:
                type += "missing-attachment";
                title = TitleAttachmentsMismatch;
                instance = "submission";
                break;
            case ProblemTypeEnum.AttachmentMismatch:
                type += "attachments-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.EncryptionIssue:
                type += "encryption-issue";
                title = TitleEncryptionIssue;
                instance = "metadata";
                break;
            case ProblemTypeEnum.HashMismatch:
                type += "hash-mismatch";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.IncorrectAuthenticationTag:
                type += "incorrect-authentication-tag";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.InvalidEventLog:
                type += "invalid-event-log";
                title = TitleEventLogInconsistent;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingAuthenticationTag:
                type += "missing-authentication-tag";
                title = TitleAuthenticationTagMissing;
                instance = "submission";
                break;
            case ProblemTypeEnum.MissingData:
                type += "missing-data";
                title = TitleMissingData;
                instance = "metadata";
                break;
            case ProblemTypeEnum.MissingSchema:
                type += "missing-schema";
                title = TitleMissingSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.ServiceMismatch:
                type += "service-mismatch";
                title = TitleServiceMismatch;
                instance = "metadata";
                break;
            case ProblemTypeEnum.SchemaViolation:
                type += "schema-violation";
                title = TitleSchemaViolationData;
                instance = "data";
                break;
            case ProblemTypeEnum.SyntaxViolation:
                type += "syntax-violation";
                title = TitleSyntaxViolation;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedDataSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedDataSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedMetaSchema:
                type += "unsupported-schema";
                title = TitleUnsupportedSchema;
                instance = "metadata";
                break;
            case ProblemTypeEnum.UnsupportedService:
                type += "unsupported-service";
                title = TitleUnsupportedService;
                instance = "metadata";
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(problemType), problemType, null);
        }

        this.detail = detail;
    }


    public Problems(ProblemTypeEnum problemType, string title, string detail,
        string problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        this.instance = problemInstance;
    }
    public Problems(ProblemTypeEnum problemType, string title, string detail,
        ProblemInstanceEnum problemInstance) : this(problemType, detail) {
        this.title = title ?? problemType switch {
            ProblemTypeEnum.InvalidEventLog => TitleEventLogInconsistent,
            _ => problemType.ToString()
        };

        this.instance = problemInstance switch {
            ProblemInstanceEnum.Metadata => "metadata",
            ProblemInstanceEnum.Submission => "submission",
            ProblemInstanceEnum.Data => "data",
            _ => throw new ArgumentOutOfRangeException(nameof(problemInstance), problemType, null)
        };
    }

    public static readonly Problems AttachmentsMismatch =
        new Problems(
            ProblemTypeEnum.AttachmentMismatch,
            "Fehlerhafte Anlagen-Liste",
            "Die Liste der Anlagen in Submission und Event-Log stimmt nicht überein.",
            ProblemInstanceEnum.Metadata);

    public static readonly Problems EncryptionIssue =
        new Problems(
            ProblemTypeEnum.EncryptionIssue,
            "Entschlüsselungs-Fehler",
            "Der Schlüssel {kid} ist nicht der zu diesem Zweck vorgesehene Schlüssel.",
            ProblemInstanceEnum.Metadata
        );

    public static readonly Problems IncorrectAuthenticationTag =
        new Problems(
            ProblemTypeEnum.IncorrectAuthenticationTag,
            "Authentication-Tag ungültig",
            "Das Authentication-Tag des Metadatensatzes ist ungültig.",
            ProblemInstanceEnum.Metadata
        );

    public static readonly Problems InvalidEventLog =
        new Problems(
            ProblemTypeEnum.InvalidEventLog,
            "Inkonsistentes Event-Log",
            "Das Event-Log ist inkonsistent, da es nicht genau ein Event 'submit-submission' enthält.",
            ProblemInstanceEnum.Submission
        );

    public static readonly Problems MissingAuthenticationTag =
        new Problems(
            ProblemTypeEnum.MissingAuthenticationTag,
            "Fehlende Authentication-Tags",
            "Das Event 'submit-submission' enthält keine Authentication-Tags.",
            ProblemInstanceEnum.Submission
        );

    public static readonly Problems MissingSchema = new Problems(
        ProblemTypeEnum.MissingSchema,
        "Schema-Referenz fehlt",
        "Die Schema-Referenz fehlt im Metadatensatz.",
        ProblemInstanceEnum.Metadata
    );

    public static readonly Problems SchemaViolation = new Problems(
        ProblemTypeEnum.SchemaViolation,
        "Schema-Fehler",
        "Der Metadatensatz ist nicht schema-valide.",
        ProblemInstanceEnum.Metadata
    );

    public static readonly Problems SyntaxViolation = new Problems(
        ProblemTypeEnum.SyntaxViolation,
        "Syntax-Fehler",
        "Der Metadatensatz ist kein valides JSON.",
        ProblemInstanceEnum.Metadata
    );

    public static readonly Problems UnsupportedSchema = new Problems(
        ProblemTypeEnum.UnsupportedMetaSchema,
        "Metadatenschema nicht unterstützt",
        "Die angegebene Metadatenschema-URI ('$schema') ist keines der unterstützten Metadatenschemas.",
        ProblemInstanceEnum.Metadata
    );


    [JsonProperty("instance", NullValueHandling = NullValueHandling.Ignore)]
    public string instance { get; set; }

    [JsonProperty("detail", NullValueHandling = NullValueHandling.Ignore)]
    public string detail { get; set; }

    [JsonProperty("title", NullValueHandling = NullValueHandling.Ignore)]
    public string title { get; set; }

    // [JsonProperty("description", NullValueHandling = NullValueHandling.Ignore)]
    // public string? description { get; set; }

    [JsonProperty("type", NullValueHandling = NullValueHandling.Ignore)]
    public string type { get; set; }

    // [JsonProperty("items", NullValueHandling = NullValueHandling.Ignore)]
    // public Items? items { get; set; }
}
